using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices;


namespace AMO_Tutorial
{
    class AMO_Tutorial
    {
        static void Main(string[] args)
        {
            Server serv = new Server();
            serv.Connect("local");
            if (serv.Connected)
            {
                Database dbs = serv.Databases.FindByName("NewDB");
                if (dbs != null)
                {
                    dbs.Drop(DropOptions.AlterOrDeleteDependents);
                }
                dbs = serv.Databases.Add("NewDB");
                dbs.Update();
                Console.WriteLine("New DB wurde erstellt, zum Erstellen der DataSource besttigen Sie mit ENTER");
                Console.ReadLine();
                DataSource ds= erstellenDatasource(dbs);
                Console.WriteLine("Die DataSource wurde erstellt, zum Erstellen der DataSourceView besttigen Sie mit ENTER");
                Console.ReadLine();
                DataSourceView dsv= erstellenDatasourceView(dbs,ds);
                Console.WriteLine("Die DataSourceView wurde erstellt, zum Erstellen der Dimension besttigen Sie mit ENTER");
                Console.ReadLine();
                erstellenDimension( dbs, dsv);
                Console.WriteLine("Die Dimension wurde erstellt, zum Erstellen des Cubes besttigen Sie mit ENTER");
                Console.ReadLine();
                erstellenCube(dbs, dsv);
                Console.WriteLine("Der Cube wurde erstellt, zum Erstellen der MiningStructure besttigen Sie mit ENTER");
                Console.ReadLine();
                MiningStructure ms = erstellenMiningStructure(dbs,dsv);
                Console.WriteLine("Die Mining Structure wurde erstellt, zum Erstellen des MiningModel besttigen Sie mit ENTER");
                Console.ReadLine();
                erstellenMiningModel(ms);
               dbs.Process();
               Console.WriteLine("Fertig");
               Console.ReadLine();
            }
            serv.Disconnect();
        }

        static DataSource erstellenDatasource(Database dbs)
        {
            //Erstellen einer DataSource 
            DataSource ds = dbs.DataSources.Add("NewDS");
            ds.ConnectionString = "Provider=SQLNCLI10.1;Data Source=Biss2008;Integrated Security=SSPI;Initial Catalog=AdventureWorksDW2008";
            ds.Update();
            return ds;
        }
        static DataSourceView erstellenDatasourceView(Database dbs, DataSource ds)
        {
            //erstellen einer DataSourceView
            DataSourceView dsv = dbs.DataSourceViews.Add("NewDSV");
            dsv.DataSourceID = ds.ID;

            //Das Schema der DataSourceView erzeugen
            dsv.Schema = new System.Data.DataSet();

            //Tabellen ins Schema einfgen 
            System.Data.OleDb.OleDbConnection conn = new System.Data.OleDb.OleDbConnection(ds.ConnectionString);
            conn.Open();
            System.Data.OleDb.OleDbCommand cmd;
            System.Data.OleDb.OleDbDataAdapter adapter;
            System.Data.DataTable[] tbls;
            System.Data.DataColumn col;



           

            //Customer Tabelle einbinden
            cmd = new System.Data.OleDb.OleDbCommand("DimCustomer", conn);
            cmd.CommandType = System.Data.CommandType.TableDirect;
            adapter = new System.Data.OleDb.OleDbDataAdapter(cmd);
            tbls = adapter.FillSchema(dsv.Schema, System.Data.SchemaType.Mapped, "DimCustomer");
            //tbls[0].ExtendedProperties.Add("TableType", "Table");
            //tbls[0].ExtendedProperties.Add("DbTableName", "DimCustomer");
            //tbls[0].ExtendedProperties.Add("FriendlyName", "Customer");
            //tbls[0].ExtendedProperties.Add("DbSchemaName", "dbo");
            col= tbls[0].Columns.Add("FullName", typeof(string));
           //col.ExtendedProperties.Add("DbColumnName","FullName");
            //col.ExtendedProperties.Add("isLogical", "true");
            col.ExtendedProperties.Add("ComputedColumnExpression", "LastName + ',' + FirstName");
            //col.MaxLength = 150;
            System.Data.DataColumn pk_cust=tbls[0].Columns["CustomerKey"];
             
            //Faktentabelle einbinden
            cmd = new System.Data.OleDb.OleDbCommand("FactInternetSales", conn);
            cmd.CommandType = System.Data.CommandType.TableDirect;
            adapter = new System.Data.OleDb.OleDbDataAdapter(cmd);
            tbls = adapter.FillSchema(dsv.Schema, System.Data.SchemaType.Mapped, "FactInternetSales");
            //tbls[0].ExtendedProperties.Add("TableType", "Table");
            //tbls[0].ExtendedProperties.Add("DbTableName", "FactInternetSales");
            //tbls[0].ExtendedProperties.Add("FriendlyName", "InternetSales");
            //tbls[0].ExtendedProperties.Add("DbSchemaName", "dbo");
            System.Data.DataColumn fk_cust = tbls[0].Columns["CustomerKey"];
            
            
            //Beziehung zwischen Customer und InternetSales ins Schema schreiben 
            dsv.Schema.Relations.Add(pk_cust,fk_cust);

            conn.Close();
            dsv.Update();
            return dsv;
        }
        static void erstellenDimension(Database dbs, DataSourceView dsv)
        {
            //Dimension erstellen
            Dimension dim = dbs.Dimensions.Add("Customer");
            dim.Type = DimensionType.Customers;
            //Der Dimension eine DataSourceVies zuweisen

            dim.Source = new DataSourceViewBinding(dsv.ID);
            //Speicherart festlegen
            dim.StorageMode = DimensionStorageMode.Molap;
            //Der Dimension ein Attribut zuweisen 
            DimensionAttribute att;
            att = dim.Attributes.Add("Name");
            att.Usage = AttributeUsage.Key;
            att.KeyColumns.Add(new DataItem("DimCustomer", "CustomerKey"));
            att.NameColumn = new DataItem("DimCustomer", "FullName");
            dim.Update();
           dim.Process();
        }
        static void erstellenCube(Database dbs,DataSourceView dsv)
        {
           //Cube erstellen
            Cube cub = dbs.Cubes.Add("Simple");
            cub.Source = new DataSourceViewBinding("NewDSV");
            cub.StorageMode = StorageMode.Molap;

            //Dimension Customer hinzufgen
            CubeDimension cubDim=cub.Dimensions.Add("Customer");

            //MeasureGroup erstellen
            MeasureGroup mg = cub.MeasureGroups.Add("InternetSalesGroup");
            mg.StorageMode = StorageMode.Molap;

            //Measure hinzufgen Herzu wird der Name, der im Schema vorhandenen Tabelle genutzt) 
            Measure me = mg.Measures.Add("OrderQuantity");
            me.AggregateFunction = AggregationFunction.Sum;
            me.Source = new DataItem("FactInternetSales", "OrderQuantity", System.Data.OleDb.OleDbType.Integer);

            //CubeDimension cubDim = cub.Dimensions.FindByName("Customer");
            //Verbindung zwischen Measure und Dimensions Schlsseln herstellen 
            //Hierzu werden wird auf MeasureGroup Ebene eine Dimension gebildet die zwischen den IDs mapped
            RegularMeasureGroupDimension mgDim = mg.Dimensions.Add(cubDim.ID);
            MeasureGroupAttribute mga = mgDim.Attributes.Add(cubDim.Dimension.Attributes.FindByName("Name").ID);
            mga.Type = MeasureGroupAttributeType.Granularity;
            mga.KeyColumns.Add(new DataItem("FactInternetSales", "CustomerKey"));

            // Partition bilden
            Partition pa = mg.Partitions.Add("P1");
            pa.StorageMode = StorageMode.Molap;
            pa.Source = new QueryBinding(cub.Parent.DataSources[0].ID, "Select * FROM dbo.FactInternetSales");

            //Cube auf den Server schreiben
            cub.Update(UpdateOptions.ExpandFull);
        }
        static public MiningStructure erstellenMiningStructure(Database dbs, DataSourceView dsv)
        {
            MiningStructure ms = dbs.MiningStructures.Add("CustomerDescionTree_Structure");
            ScalarMiningStructureColumn smsc;

            ms.Source = new DataSourceViewBinding(dsv.ID);
            ms.CaseTableName = "DimCustomer";

            // Von der Tabelle DimCustomer werden einige Spalten in die Minin Stucture bernommen
            // CustomerKey
            smsc = new ScalarMiningStructureColumn("CustomerKey");
            smsc.IsKey = true;
            smsc.Content = "Key";
            smsc.KeyColumns.Add("DimCustomer", "CustomerKey", System.Data.OleDb.OleDbType.Integer);
            ms.Columns.Add(smsc);

            // YearlyIncome
            smsc = new ScalarMiningStructureColumn("YearlyIncome");
            smsc.IsKey = false;
            smsc.Content = "Discrete";
            smsc.KeyColumns.Add("DimCustomer", "YearlyIncome", System.Data.OleDb.OleDbType.WChar);
            ms.Columns.Add(smsc);


            // MaritalStatus
            smsc = new ScalarMiningStructureColumn("MaritalStatus");
            smsc.IsKey = false;
            smsc.Content = "Discrete";
            smsc.KeyColumns.Add("DimCustomer", "MaritalStatus", System.Data.OleDb.OleDbType.WChar);
            ms.Columns.Add(smsc);


            //Children
            smsc = new ScalarMiningStructureColumn("Children");
            smsc.IsKey = false;
            smsc.Content = "Continuous";
            smsc.KeyColumns.Add("DimCustomer", "TotalChildren", System.Data.OleDb.OleDbType.Integer);
            ms.Columns.Add(smsc);

            ms.Update();

            return ms;
        }

        static public void erstellenMiningModel(MiningStructure ms)
        {
            MiningModelColumn mc;
            MiningModel mm = ms.MiningModels.Add("CustomerDescionTree_Modell");

            mm.Algorithm = "Microsoft_Decision_Trees";
            mm.AlgorithmParameters.Add("COMPLEXITY_PENALTY", 0.3);

            mc = new MiningModelColumn("Customer ID","Customer ID");
            mc.SourceColumnID = ms.Columns["CustomerKey"].ID;
            mc.Usage = "Key";
            mm.Columns.Add(mc);

            mc = new MiningModelColumn("Yearly income", "Yearly income");
            mc.SourceColumnID = ms.Columns["YearlyIncome"].ID;
            mc.Usage = "Input";
            mm.Columns.Add(mc);

            mc = new MiningModelColumn("MaritalStatus", "MaritalStatus");
            mc.SourceColumnID = ms.Columns["MaritalStatus"].ID;
            mc.Usage = "PredictOnly";
            mm.Columns.Add(mc);

            mc = new MiningModelColumn("Children", "totalChidren");
            mc.SourceColumnID = ms.Columns["Children"].ID;
            mc.Usage = "Input";
            mm.Columns.Add(mc);

            mm.Update();

          // return mm;
        }

    }
}
